"""

==========================
Lemon - Python Web Modules
==========================

:Summary: Fully object-orientated Python modules designed to make web programming easy and intuitive.
:Licence: GNU GPL
:Author: James Gardner

The documentation for the Lemon modules can be found in the ``doc`` directory of the source distribution or online at http://lemon.sourceforge.net/.

----

This module has the following public variables
        
config
    A ConfigParser object which contains the variables listed in Lemon.ini
    
    Useage::
    
        import lemon
        # Return the value of cookie.domain in the session section
        lemon.config.get('session','cookie.domain') 
        # Set the value of cookie.domain in the session section
        lemon.config.set('session','cookie.domain') 
                    
    Usually you would only get the variable.

cgi
    dictionary containing key value pairs from the QUERY_STRING. All values are strings.
    
    Useage
    
    For the URL http://www.example.com/test.py?key=5&key2=hello::
    
        import lemon
        print lemon.get['key']
        print lemon.get['key2']
    
    Would produce::
    
        5
        hello

"""
__docformat__ = "restructuredtext en"
__author__ = "James Gardner [email: james at xecos.com]"
__date__ = "02 July 2003"
__version__ = "Version 0.1 for LEMON 0.2.0-Pre-Alpha"[-15:]
__credits__ = "Guido van Rossum, for an excellent programming language."

#__all__ = ['auth']

try:
    True
except:
    True = 1
    False = 0

class LemonError(Exception):
    """Raised when a general Error in a lemon module occurs
    Attributes:
        message -- explanation of what the specific error is.

    Useage:
        raise LemonError('There was an error')

        try:
            ...
        except LemonError, e:
            print e.message
            
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

import sys, os
sys.path.append(__path__[0])
sys.path.append('../../../MySQLdb')
external = 'external'
if sys.platform.startswith('freebsd'):
    external = 'external_freebsd4'
d = [os.path.join(__path__[0], external)]
for path in sys.path:
    d.append(path)
    #d.insert(0, path)
sys.path = d
del d

import cgi as cgi_proper
get = cgi_proper.parse(environ=os.environ, keep_blank_values=True, strict_parsing=False)
"""Cool Val"""
for k, v in get.items():
    if len(get[k]) == 1:
        get[k] = str(v[0])

cgi = get


#loadConfig()
    
def findFile(filename):
    """Utility function which searched the directories listed in sys.path for the file specified in filename.
    If the file exists the complete path is returned. 
    
    Note that the directories are searched in the order they appear in sys.path so if you want to choose a specific file it is better not to use this function 
    because a similar file in a directory earlier in sys.path will be returned first."""
    for p in sys.path:
        if os.path.exists(os.path.join(p,filename)):
            return os.path.join(p,filename)
    return filename
    
    
#~ def loadConfig():
        
    #~ #from config import config
    #~ #configDict = config.options
    #~ from conf import options
    #~ import StringIO
    #~ file = StringIO.StringIO(options)
    
    #~ config = ConfigParser.ConfigParser({'prefix':sys.prefix,'version':__version__})
    #~ config.readfp(file)                         # Load the default values from 'conf.py' options string.
    #~ config.read(['Lemon.ini'])  # Load other values from 'Lemon.ini'.
    
    #~ "Basic error checking on the values from the config file."
    #~ sections = []
    #~ sections.append(
                        #~ (
                            #~ 'db',       (
                                            #~ 'type',
                                            #~ 'database',
                                            #~ 'user',
                                            #~ 'password',
                                            #~ 'host',
                                            #~ 'port',
                                            #~ 'socket',
                                            #~ 'gadfly.dir',
                                            #~ 'gadfly.colTypes',
                                        #~ ),
                            #~ 'session',  (
                                            #~ 'name',
                                            #~ 'expires',
                                            #~ 'cleanupProbablity',
                                            #~ 'entropySeed',
                                            #~ 'storage',
                                            #~ 'db.table',
                                            #~ 'db.sessionID',
                                            #~ 'db.app',
                                            #~ 'db.itemKey',
                                            #~ 'db.itemValue',
                                            #~ 'cookie.lifetime',
                                            #~ 'cookie.path',
                                            #~ 'cookie.domain',
                                            #~ 'cookie.comment',
                                            #~ 'cookie.secure',
                                            #~ 'cookie.max-age',
                                        #~ ),
                            #~ 'auth',  (
                                            #~ 'expire',
                                            #~ 'idle',
                                            #~ 'autoLogin',
                                        #~ ),
                            #~ 'mail',  (
                                            #~ 'method',
                                            #~ 'smtp',
                                            #~ 'sendmail',
                                            #~ 'from',
                                        #~ ), 
                        #~ )
                    #~ )
    
    #~ missingVars = []
    #~ for t in sections:
    
        #~ section = t[0]
        #~ options = t[1]
        #~ for option in options:
            #~ if not config.has_option(section,option):
                #~ missingVars.append("["+section+"]->"+option)
    #~ if len(missingVars) > 0:
        #~ raise LemonError("The following values have not been set in the Lemon.ini configuration file: %s"%' ,'.join(missingVars))

def getHTTPHeader(type='text/html'):
    """Generates the Content-type header for a particular mime type such as 'text/html'."""
    return "Content-type: %s\n\n" % (type)
    
def header(type='text/html'):
    return getHTTPHeader(type)
    
def wrap(text, width):
    """
    A word-wrap function that preserves existing line breaks
    and most spaces in the text. Expects that existing line
    breaks are posix newlines (\n).
    """
    return reduce(lambda line, word, width=width: '%s%s%s' %
                  (line,
                   ' \n'[(len(line[line.rfind('\n')+1:])
                         + len(word.split('\n',1)[0]
                              ) >= width)],
                   word),
                  text.split(' ')
                 )
        
def mail(**params):#msg, me, to, subject, smtpServer, blind=False
    """Function to send a text only email via SMTP.
        msg         - Text of the message.
        to          - A list of recipient addresses in the form: addr@addr.com separated by commas.
        subject     - Email subject.
        smtp        - SMTP server address.
        sendmail    - Sendmail path.
        method      - Which method to use if smtp or sendmail aren't specified.
        blind       - Whether to send the emails blind or not.
        reply       - The name and address of the person sending the email in the form: "sender name <addr@addr.com>"
               
        
        WARNING: This function probably doesn't work yet. Don't use it!!!
    """

   
    # imports
    import StringIO
    import smtplib
    from email.MIMEText import MIMEText
    from email.MIMEBase import MIMEBase
    import email.Utils
    
    if not params.has_key('blind'):
        params['blind'] = False
    if params['blind'] not in [True, False]:
        raise LemonError("'blind' can only be True or False.")
    # Make sure the essential attributes are there
    for p in ['msg', 'to', 'subject']:
        if not params.has_key(p):
            raise LemonError("You must specify the attribute '%s' to send an email."%p)
            
        
    # Make sure they are stings
    for p in ['msg', 'subject']:
        if type(params[p]) <> type(''):
            raise LemonError("The attribute '%s' must be a string."%p)

    #~ # Make sure the sender address is valid
    #~ params = addToDict({
                                #~ 'reply':config.get('mail','reply'),
                                #~ 'sender':config.get('mail','sender'),
                                #~ 'method':config.get('mail','method'),
                            #~ }, params)
                            
    #if not addr[0] or not addr[1]:
    #    raise LemonError("'%s' is not a valid name address pair. Should be of the form 'Sender Name <test@example.com>'."%params['sender'])
        
    if params['method'] not in ['sendmail','smtp']:
        raise LemonError("The method parameter cannot be '%s'."%params['method'])
        
    #~ if params.has_key('smtp') and params['method'] == 'sendmail':
        #~ raise LemonError("You cannot specify the method as 'sendmail' if you are sending by smtp.")
        
    #~ if params.has_key('sendmail') and params['method'] == 'smtp':
        #~ raise LemonError("You cannot specify the method as 'smtp' if you are sending by sendmail.")
        
    # XXX

    #~ if params.has_key('smtp'):
        #~ params['method'] = 'smtp'
        
    #~ if params.has_key('sendmail'):
        #~ params['method'] = 'sendmail'

                            
    # Open a plain text file for reading.  For this example, assume that
    # the text file contains only ASCII characters.
    fp = StringIO.StringIO(params['msg'])
    if not params.has_key('type'):
        msg = MIMEText(fp.read())
    else:
        msg = MIMEBase('text',params['type'])
        msg.set_payload(fp.read())
    fp.close()
    
    msg['From'] = params['reply']
    msg['Subject'] = params['subject']
    if type(params['to']) is type(''):
        params['to']=[params['to']]
            
    if params['method'] == 'sendmail':

            
        if params['blind'] is True:
            msg['Bcc'] = ', '.join(params['to'])
        else:
            msg['To'] = ', '.join(params['to'])
            
        import os
        if not os.path.exists(params['sendmail']):
            raise LemonError("The path '%s' doesn't exist. Please check the location of sendmail."%params['sendmail'])
        fp = os.popen(params['sendmail']+" -t", 'w')
        fp.write(msg.as_string())
        error = fp.close()
        if error:
            raise LemonError("Error sending mail: Sendmail Error '%s'."% error)
            
    elif params['method'] == 'smtp':

        # Make sure the addresses in 'to' are valid:
        #addrs = email.Utils.getaddresses(params['to']) 


        #to = []
        #print email.Utils.formataddr(('james gardner','james@jimmyg.org')) 
        #for t in addrs:
            #print t
            #to.append(t[1])
   
        if params['blind'] == True:
            msg['To'] = ''
        else:
            msg['To'] = ', '.join(params['to'])
        
        # Send the message via our own SMTP server, but don't include the
        # envelope header.
        s = smtplib.SMTP()
        #s.set_debuglevel(100) 

        s.connect(params['smtp'])
        
        result = s.sendmail(params['reply'], params['to'], msg.as_string())

        s.close()
        
        if result:
            for r in result.keys():
                error+= "Error sending to"+ str(r)
                rt = result[r]
                error+= "Code"+ str(rt[0])+":"+ str(rt[1])
            raise LemonError("Error sending mail: %s"% error)

def cgiAsDict(form):
    """Takes a cgi.FieldStorage() item and retuns a simple dictionary for use in other lemon functions."""
    dict = {}
    for k in form.keys():
        dict[k] = form.getvalue(k,'')
    return dict
    
def addToDict(dict1, dict2, overWrite=True):
    """Adds the information in the dictionary 'dict2' to the entries in the dictionary 'dict1'.
    If 'overWrite' is True the values in 'dict2' overwrite the values in 'dict1', otherwise only the keys not 
    already in 'dict1' are added."""
    
    dict={}
    if overWrite == True:
        for k,v in dict1.items():
            dict[k] = v
            
    if type(dict2) == type([]) or type(dict2) == type(()):
        for k,v in dict2:
            dict[k] = v
    else:
        for k,v in dict2.items():
            dict[k] = v
    
    if overWrite == False:
        for k,v in dict1.items():
            dict[k] = v
            
    return dict
    
def removeFromDict(dict1, keys):
    """Removes all keys in 'dict1' specified in the list or dictionary 'keys'."""
    dict = {}
    for k,v in dict1.items():
        if k not in keys:
            dict[k] = v
    return dict
    
def keepInDict(dict1, keys):
    """Removes all keys in 'dict1' NOT specified in the list or dictionary 'keys'."""
    dict = {}
    for k,v in dict1.items():
        if k in keys:
            dict[k] = v

    return dict
    
def dictAsURLPairs(stickyData):
    """Changes a dictionary such as lemon.get into a query string."""
    import urllib
    url = ''
    for k, v in stickyData.items():
        
        if type(v) <> type([]):
            url += urllib.quote_plus(str(k))+'='+urllib.quote_plus(str(v))+'&'
        else:
            for item in v:
                url += urllib.quote_plus(k)+'='+urllib.quote_plus(str(item))+'&'
    return url

def dictAsHiddenFields(stickyData):
    """Changes a dictionary such as lemon.get into a string containing hidden fields HTML."""
    fields = ''
    for k,v in stickyData.items():
        if type(v) <> type([]):
            fields += '<input type="hidden" name="%s" value="%s">\n' % (str(k),str(v)) # No _plus
        else:
            for item in v:   
                fields += '<input type="hidden" name="%s" value="%s">\n' % (str(k),str(item)) # No _plus
    return fields
        
def addZeros(num,s):
    "Private function to add an appropriate number of zeros to s such that len(s) is num."
    s = str(s)
    while( len(s) < num ):
        s = '0'+s
    return s
    
def allNone(value):
    allNone = True
    for n in value:
        if type(n) <> type(None):
            allNone = False
            break
    return allNone
                    
def escape(value):
    val = str(value)
    return val.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;').replace('\n','<br>')

def unescape(value):
    return value.replace('&amp;','&').replace('&lt;','<').replace('&gt;','>')

