"""PHP-style authorisation functions.

Version:

    File Version 0.1 for LEMON 0.2
    
Usage:

    session = lemon.session.SessionDB(self.database, appName)  # Retrieve the session or create a new one
    auth    = lemon.auth.AuthDB(session)                # Create auth object from the session information
    
    if auth.isValid():                                  # See if the User is logged on else present login form.
        user = a.getUser()                              # Return a user object containing user's details.
        user.Password                                   # Access the data via attributes or dict style.
        user.getStatus()                                # Retrieve the entry for the appName column.

ToDO:

    addUser, RemoveUser etc.
    gadfly
    create the database if not already there?? 
    definately need config file similar to php.ini
"""

__author__ = "James Gardner [email: james at xecos.com]"
__date__ = "02 July 2003"
__version__ = "Version 0.1 for LEMON 0.2"[-3:]
__credits__ = "Guido van Rossum, for an excellent programming language."

import time
import lemon.auth.manager
from lemon.auth import AuthError

class DB: # XXX (lemon.auth.manager.Base):
    "Database interface to the authorisation code."

    def __init__(self, conn):
        #self.conn = conn
        #self.database = conn.cursor(autoConvert=True)
        self.database = conn
        
    def addApplication(self, name):
        if self.applicationExists(name):
            raise AuthError("The '%s' application already exists."%name)
        else:
            self.database.insert('AuthApps', 'Name', name)
            
    def removeApplication(self, name):
        if not self.applicationExists(name):
            raise AuthError("The '%s' application doesn't exist in the database."%name)
        else:
            self.database.delete('Auth',where="App='"+name+"'")
            self.database.delete('AuthApps',where="Name='"+name+"'")
            return 1
            
    def applicationExists(self,name):
        rows = self.database.select('Name','AuthApps', where="Name='"+name+"'")
        if rows:
            return 1
        else:
            return 0
        
    def userExists(self, user):
        rows = self.database.select('User','AuthUsers', where="User='"+user+"'")
        if rows:
            return True
        return False
        
    def addUser(self, user, password, firstname='', surname='', email=''):
        if self.userExists(user):
            raise AuthError('That user already exists.')
        else:
            self.database.insert('AuthUsers',('User', 'Password', 'Firstname', 'Surname', 'Email'),(user,password,firstname,surname,email))
            return True

    def setUserLevel(self, user, app, level):
        if self.userExists(user):
            if self.applicationExists(app):
                rows = self.database.select('AccessLevel','Auth',where="User='"+user+"' and App='"+app+"'")
                if rows:
                    self.database.update('Auth', ('User','App','AccessLevel'), (user,app,int(level)), where="User='"+user+"' and App='"+app+"'")
                else:
                    self.database.insert('Auth', ('User','App','AccessLevel'), (user,app,int(level)))
                return True
            else:
                raise AuthError("The '%s' application doesn't exist in the database."%name)
        else:
            raise AuthError('That user doesn\'t exist.')
        
    def getUserLevel(self, user, app):
        if self.userExists(user):
            rows = self.database.select('AccessLevel','Auth',where="User='"+user+"' and App='"+app+"'")
            if rows:
                return rows[0][0]
            else:
                return 0
        else:
            raise AuthError('That user doesn\'t exist.')
            
    def setUserFirstname(self, user, firstname):
        if self.userExists(user):
            self.database.update('AuthUsers', ('Firstname',), (firstname,), where="User='"+user+"'")
        else:
            raise AuthError('That user doesn\'t exist.')

    def setUserSurname(self, user, surname):
        if self.userExists(user):
            self.database.update('AuthUsers', ('Surname',), (surname,), where="User='"+user+"'")
        else:
            raise AuthError('That user doesn\'t exist.')

    def setUserEmail(self, user, email):
        if self.userExists(user):
            self.database.update('AuthUsers', ('Email',), (email,), where="User='"+user+"'")
        else:
            raise AuthError('That user doesn\'t exist.')
            
    def setUserPassword(self, user, password):
        if self.userExists(user):
            self.database.update('AuthUsers', ('Password',), (password,), where="User='"+user+"'")
        else:
            raise AuthError('That user doesn\'t exist.')
            
    def removeUser(self, user):
        if self.userExists(user):
            self.database.delete('AuthUsers',where="User='"+user+"'")
            self.database.delete('Auth',where="User='"+user+"'")
        else:
            raise AuthError("The user '%s' doesn't exist."%(user))
        
    def createTables(self, autoCommit=False): # autoCommit needed for the Gadfly types information to be added.
        errors = []
        
        # Auth Users
        if not self.database.tableExists('AuthUsers'):
            self.database.create   (
                                        'AuthUsers',
                                        (    
                                            ('User',     self.database.STRING ),
                                            ('Password', self.database.STRING ),
                                            ('Firstname',self.database.STRING ),
                                            ('Surname',  self.database.STRING ),
                                            ('Email',    self.database.STRING ),
                                        )
                                    )
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append('The AuthUsers table already exists.')
    
        # Auth
        if not self.database.tableExists('Auth'):
            self.database.create   (
                                        'Auth',
                                        (    
                                            ('User',     self.database.STRING ),
                                            ('App',      self.database.STRING ),
                                            ('AccessLevel',    self.database.INTEGER),
                            
                                        )
                                    )
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append('The Auth table already exists.')
            
        # AuthApps
        if not self.database.tableExists('AuthApps'):
            self.database.create   ( 
                                        'AuthApps',
                                        (    
                                            ('Name',     self.database.STRING ),
                                        )
                                    )
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append('The AuthApps table already exists.')

        return errors
    
    def dropTables(self, autoCommit=False):
        errors = []
        if self.database.tableExists('Auth'):
            self.database.drop('Auth')
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append("The Auth table doesn't exist.")
        if self.database.tableExists('AuthApps'):
            self.database.drop('AuthApps')
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append("The AuthApps table doesn't exist.")
        if self.database.tableExists('AuthUsers'):
            self.database.drop('AuthUsers')
            if autoCommit:
                raise Exception('Autocommit not implemented')
                #self.conn.commit()
        else:
            errors.append("The AuthUsers table doesn't exist.")
        return errors
            
    def tablesExist(self):
        if self.database.tableExists('Auth') and self.database.tableExists('AuthUsers') and self.database.tableExists('AuthApps'):
            return True
        return False