"""Lemon DB interface to the MySQLdb cursor.

Version:

    File Version 0.1 for LEMON 0.2
    
Description:
    
    Module containing the mysql cursor.
"""

__author__ = "James Gardner [email: james at xecos.com]"
__date__ = "02 July 2003"
__version__ = "Version 0.1 for LEMON 0.2"[-3:]
__credits__ = "Guido van Rossum, for an excellent programming language."


import MySQLdb
import base
import lemon.db

class mysqlConnection(base.baseConnection):
    def __init__(self, database, user='', password='', host='', socket='', port=''):
        'Set up the database connection'
        args = dict(
            db=database, 
            user=user, 
            passwd=password, 
            host=host, 
        )
        if port:
            args['port'] = int(port)
        if socket:
            args['unix_socket'] = socket
        self.connection = MySQLdb.connect(**args) 
        
    def cursor(self, autoExecute=True, fetchMode='tuple', autoConvert=False, convertMode='description', colTypesName='ColTypes', types={}):
        "Return our custom cursor rather than the default connection cursor."
        return mysqlCursor(self.connection.cursor(), autoExecute, fetchMode, autoConvert, convertMode, colTypesName, types)
        
class mysqlCursor(base.baseCursor):


    def setupOptions(self):
        return [
                    [True, False],              # autoExecute
                    ['tuple','dict','dtuple'],  # fetchMode
                    [True, False],              # autoConvert
                    ['table', 'description'],   # convertMode
                ]
                
                
#
# Encoding Functions
#

    def setupFields(self):
        
        self.CHAR       = (254,)
        self.STRING     = (253,)
        self.BINARY     = (252,)
        self.INTEGER    = (3,5)
        self.FLOAT      = (4,)
        self.DATE       = (10,)
        self.TIME       = (11,)
        self.DATETIME   = (12,)
        
        self.fieldTypes =   {
                                self.CHAR       :"CHAR(1)",
                                self.STRING     :"VARCHAR(255)",
                                self.BINARY     :"MEDIUMBLOB",
                                self.INTEGER    :"INT(11)",
                                self.FLOAT      :"FLOAT",
                                self.DATE       :"DATE",
                                self.TIME       :"TIME",
                                self.DATETIME   :"DATETIME",
                            }

