"""Implementation of a database storage driver for the session module."""

import lemon, pickle, StringIO, lemon, lemon.session, time
import base

class DB(base.Base):
    """The main session class for database storage of sessions.
    See the information in lemon.session.start() for details of how to use it. The class should not be called 
    directly becuase error-checking occurs in lemon.session.start()."""
        
    def __init__(self, app, expires, sessionID, cookie, seed, cleanup, database, table):
        """Initialise the class with the parameters as specified in the lemon.session.start() function."""
        
        self.params={} # Store all member variables in the params dictionary.
        
        # Set up options allowed in the constructor
        if name == 'Sessions':
            raise SessionError("The app name 'Sessions' is reserved for use by the session module.")
        else:
            self.params['name'] = app
        self.params['expires'] = expires
        self.params['sessionID'] = sessionID
        self.params['cookie'] = cookie
        self.params['seed'] = seed
        self.params['cleanup'] = cleanup
        self.params['database'] = database
        self.params['cursor'] = self.params['database'].cursor(autoConvert=True)
        self.params['table'] = table
        
        self.begin()
        
    def _addSession(self, sessionID, length):
        "Create a new session setting the 'Created' and 'Accessed' variables."
        #~ self.params['cursor'].insert(   
                                        #~ self.params['table'],
                                        #~ ('SessionID', 'App', 'ItemKey','ItemValue'),
                                        #~ (self.params['sessionID'],'Sessions','Created',int(time.time())),
                                    #~ )
        self.params['cursor'].insert(   
                                        self.params['table'], 
                                        ('SessionID', 'App', 'ItemKey','ItemValue'),
                                        (self.params['sessionID'],self.params['table'],'Expires',int(time.time())+int(length)),
                                    )
        self.params['database'].commit()
                                    
    def _removeSession(self, sessionID):
        "Remove session information from the session store."
        self.params['cursor'].delete(self.params['table'], where="SessionID='"+sessionID+"'")
        self.params['database'].commit()
        
    def _cleanupSessions(self):
        "Remove all expired sessions."
        rows = self.params['cursor'].select('SessionID',self.params['table'], where="ItemKey='Created' and ItemValue < "+str(int(time.time()) - int(self.params['expires'])))      
        for row in rows:
            self.params['cursor'].delete(self.params['table'], where="SessionID='"+row[0]+"'")      
        self.params['database'].commit()
        
    def _sessionExists(self, sessionID):
        "Returns True if the session already exists, False otherwise."
        rows = self.params['cursor'].select('SessionID',self.params['table'], where="SessionID='"+sessionID+"'" )
        if rows:
            return True
        return False
        
    def _set(self, key, value, app=None, pickleValue=True):
        "Sets the variable 'key' to the value 'value' after pickling it. # XXX The app functionality is not tested"
        if not app:
            app = self.params['name']
        if pickleValue:
            out = StringIO.StringIO()
            pickle.dump(value,out)
            value = out.getvalue()
    
        rows = self.params['cursor'].select('ItemValue',self.params['table'],where="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"'")
        if len(rows) == 0:
            self.params['cursor'].insert(self.params['table'], ('SessionID','App','ItemKey','ItemValue'), (self.params['sessionID'], self.params['name'], key, value))
            self.params['database'].commit()
        else:
            self.params['cursor'].update(self.params['table'], ('ItemValue',), (value,), where="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"' and App='"+app+"'")  
            self.params['database'].commit()
    
    def _get(self, key, app=None, pickleValue=True):
        "Gets the variable 'key' from the session store."
        if not app:
            app = self.params['name']
        rows=self.params['cursor'].select(('ItemValue'),self.params['table'],where="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"' and App='"+app+"'")  
        if not rows:
            return None
        else:
            if self.params['cursor'].fetchMode in ['dict','dtuple']:
                read = StringIO.StringIO(rows[0]['ItemValue'])
            elif 'tuple' == self.params['cursor'].fetchMode:
                read = StringIO.StringIO(rows[0][0])
            else:
                raise lemon.db.SessionError("Database fetchmode '%s' not understood."%self.params['cursor'].getFetchMode())
            if pickleValue:
                return pickle.load(read)
            else:
                return read
                
    def _delete(self, key):
        "Removes the variable from the session store."
        self.params['cursor'].delete(self.params['table'],where="SessionID='"+self.params['sessionID']+"' and ItemKey='"+key+"'")  
        self.params['database'].commit()
        
        
    #~ def _updateAccessed(self):
        #~ "Update the 'Accessed' variable of the session."
        #~ self.params['cursor'].update(   
                                        #~ self.params['table'], 
                                        #~ 'ItemValue',
                                        #~ int(time.time()),
                                        #~ where="SessionID='"+self.params['sessionID']+"' and App='Sessions' and ItemKey='Accessed'",
                                    #~ )
        #~ self.params['database'].commit()
                                    

        
